package io.gitlab.jfronny.commons.serialize.gson.dsl

import io.gitlab.jfronny.gson.Gson
import io.gitlab.jfronny.gson.JsonArray
import io.gitlab.jfronny.gson.JsonObject
import io.gitlab.jfronny.gson.stream.JsonTreeWriter
import io.gitlab.jfronny.gson.stream.JsonWriter
import java.io.OutputStream
import java.io.StringWriter
import java.io.Writer
import java.nio.file.Path
import kotlin.io.path.writer

@DslMarker
@Target(AnnotationTarget.CLASS, AnnotationTarget.TYPE)
annotation class JsonDsl

fun Path.jObject(gson: Gson, content: (@JsonDsl ObjectScope).() -> Unit) = openJson(gson) { jObject(content) }
fun Path.jArray(gson: Gson, content: (@JsonDsl ArrayScope).() -> Unit) = openJson(gson) { jArray(content) }
fun Writer.jObject(gson: Gson, content: (@JsonDsl ObjectScope).() -> Unit) = openJson(gson) { jObject(content) }
fun Writer.jArray(gson: Gson, content: (@JsonDsl ArrayScope).() -> Unit) = openJson(gson) { jArray(content) }
fun OutputStream.jObject(gson: Gson, content: (@JsonDsl ObjectScope).() -> Unit) = openJson(gson) { jObject(content) }
fun OutputStream.jArray(gson: Gson, content: (@JsonDsl ArrayScope).() -> Unit) = openJson(gson) { jArray(content) }
fun jObjectString(gson: Gson, content: (@JsonDsl ObjectScope).() -> Unit): String = StringWriter().use {
    it.jObject(gson, content)
    it.toString()
}
fun jArrayString(gson: Gson, content: (@JsonDsl ArrayScope).() -> Unit): String = StringWriter().use {
    it.jArray(gson, content)
    it.toString()
}
fun jObjectTree(content: (@JsonDsl ObjectScope).() -> Unit): JsonObject = JsonTreeWriter().use {
    it.jObject(content)
    it.get()
}.asJsonObject
fun jArrayTree(content: (@JsonDsl ArrayScope).() -> Unit): JsonArray = JsonTreeWriter().use {
    it.jArray(content)
    it.get()
}.asJsonArray

fun JsonWriter.jObject(content: (@JsonDsl ObjectScope).() -> Unit) {
    beginObject()
    ObjectScope(this).content()
    endObject()
}

fun JsonWriter.jArray(content: (@JsonDsl ArrayScope).() -> Unit) {
    beginArray()
    ArrayScope(this).content()
    endArray()
}

private fun Path.openJson(gson: Gson, write: JsonWriter.() -> Unit) = writer().use { it.openJson(gson, write) }
private fun Writer.openJson(gson: Gson, write: JsonWriter.() -> Unit) = gson.newJsonWriter(this).use(write)
private fun OutputStream.openJson(gson: Gson, write: JsonWriter.() -> Unit) = writer().use { it.openJson(gson, write) }
