package io.gitlab.jfronny.muscript.gson.impl;

import io.gitlab.jfronny.gson.GsonBuilder;
import io.gitlab.jfronny.gson.TypeAdapter;
import io.gitlab.jfronny.gson.stream.JsonReader;
import io.gitlab.jfronny.gson.stream.JsonWriter;
import io.gitlab.jfronny.muscript.data.dynamic.*;
import io.gitlab.jfronny.muscript.data.dynamic.additional.*;
import io.gitlab.jfronny.muscript.gson.impl.typed.*;
import io.gitlab.jfronny.muscript.gson.impl.typed.additional.*;

import java.io.IOException;

public class DynamicSerializer extends TypeAdapter<Dynamic> {
    public final DObjectSerializer dObject = new DObjectSerializer(this);
    public final DListSerializer dList = new DListSerializer(this);
    public final DNullSerializer dNull = new DNullSerializer();
    public final DBoolSerializer dBool = new DBoolSerializer();
    public final DNumberSerializer dNumber = new DNumberSerializer();
    public final DStringSerializer dString = new DStringSerializer(this);
    public final DCallableSerializer dCallable = new DCallableSerializer();
    public final DynamicBaseSerializer dynamicBase = new DynamicBaseSerializer();
    public final DDateSerializer dDate = new DDateSerializer();
    public final DTimeSerializer dTime = new DTimeSerializer();
    public final DEnumSerializer dEnum = new DEnumSerializer();

    public void registerTo(GsonBuilder builder) {
        builder.registerTypeAdapter(Dynamic.class, this);
        builder.registerTypeHierarchyAdapter(DObject.class, dObject);
        builder.registerTypeHierarchyAdapter(DList.class, dList);
        builder.registerTypeAdapter(DNull.class, dNull);
        builder.registerTypeHierarchyAdapter(DBool.class, dBool);
        builder.registerTypeHierarchyAdapter(DNumber.class, dNumber);
        builder.registerTypeHierarchyAdapter(DString.class, dString);
        builder.registerTypeHierarchyAdapter(DCallable.class, dCallable);
        builder.registerTypeHierarchyAdapter(DynamicBase.class, dynamicBase); // Custom dynamic implementations need custom support
        builder.registerTypeAdapter(DDate.class, dDate);
        builder.registerTypeAdapter(DTime.class, dTime);
        builder.registerTypeAdapter(DEnum.class, dEnum);
    }

    @Override
    public Dynamic read(JsonReader r) throws IOException {
        return switch (r.peek()) {
            case BEGIN_OBJECT -> dObject.read(r);
            case BEGIN_ARRAY -> dList.read(r);
            case NULL -> dNull.read(r);
            case BOOLEAN -> dBool.read(r);
            case NUMBER -> dNumber.read(r);
            case STRING -> dString.read(r);
            default -> throw new IllegalStateException("Unsupported token for beginning of Dynamic: " + r.peek());
        };
    }

    @Override
    public void write(JsonWriter w, Dynamic dynamic) throws IOException {
        if (dynamic == null) w.nullValue();
        else if (dynamic instanceof DDate date) dDate.write(w, date);
        else if (dynamic instanceof DTime time) dTime.write(w, time);
        else if (dynamic instanceof DEnum enm) dEnum.write(w, enm);
        else if (dynamic.isObject()) dObject.write(w, dynamic.asObject());
        else if (dynamic.isList()) dList.write(w, dynamic.asList());
        else if (dynamic instanceof DNull n) dNull.write(w, n);
        else if (dynamic.isBool()) dBool.write(w, dynamic.asBool());
        else if (dynamic.isNumber()) dNumber.write(w, dynamic.asNumber());
        else if (dynamic.isString()) dString.write(w, dynamic.asString());
        else if (dynamic.isCallable()) dCallable.write(w, dynamic.asCallable());
        else throw new IllegalArgumentException("Unexpected dynamic type for: " + dynamic);
    }
}
