package io.gitlab.jfronny.muscript.ast.bool;

import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.BoolExpr;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.literal.BoolLiteral;

import java.io.IOException;
import java.util.stream.Stream;

public class And extends BoolExpr {
    private final BoolExpr left;
    private final BoolExpr right;

    public And(CodeLocation location, BoolExpr left, BoolExpr right) {
        super(Order.And, location);
        this.left = left;
        this.right = right;
    }

    @Override
    public Boolean get(Scope dataRoot) {
        return left.get(dataRoot) && right.get(dataRoot);
    }

    @Override
    public BoolExpr optimize() {
        BoolExpr left = this.left.optimize();
        BoolExpr right = this.right.optimize();
        if (left instanceof BoolLiteral literal) return literal.value ? right : literal(location, false);
        if (right instanceof BoolLiteral literal) return literal.value ? left : literal(location, false);
        return new And(location, left, right);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.concat(left.extractSideEffects(), right.extractSideEffects());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        parenthesize(left, writer, false);
        writer.append(" & ");
        parenthesize(right, writer, true);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof And and && left.equals(and.left) && right.equals(and.right);
    }
}
