package io.gitlab.jfronny.muscript.ast.conditional;

import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.bool.Not;
import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.BoolExpr;
import io.gitlab.jfronny.muscript.ast.NumberExpr;
import io.gitlab.jfronny.muscript.ast.literal.BoolLiteral;

import java.io.IOException;
import java.util.stream.Stream;

public class NumberConditional extends NumberExpr {
    public final BoolExpr condition;
    public final NumberExpr trueExpr;
    public final NumberExpr falseExpr;

    public NumberConditional(CodeLocation location, BoolExpr condition, NumberExpr trueExpr, NumberExpr falseExpr) {
        super(Order.Conditional, location);
        this.condition = condition;
        this.trueExpr = trueExpr;
        this.falseExpr = falseExpr;
    }

    @Override
    public Double get(Scope dataRoot) {
        return condition.get(dataRoot) ? trueExpr.get(dataRoot) : falseExpr.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        BoolExpr condition = this.condition.optimize();
        NumberExpr trueExpr = this.trueExpr.optimize();
        NumberExpr falseExpr = this.falseExpr.optimize();
        if (condition instanceof BoolLiteral literal) return literal.value ? trueExpr : falseExpr;
        if (trueExpr.equals(falseExpr)) return trueExpr;
        if (condition instanceof Not not) return new NumberConditional(location, not.inner, falseExpr, trueExpr);
        return new NumberConditional(location, condition, trueExpr, falseExpr);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return UnresolvedConditional.extractSideEffects(location, condition, trueExpr, falseExpr);
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        parenthesize(condition, writer, true);
        writer.append(" ? ");
        trueExpr.decompile(writer);
        writer.append(" : ");
        falseExpr.decompile(writer);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof NumberConditional conditional
                && condition.equals(conditional.condition)
                && trueExpr.equals(conditional.trueExpr)
                && falseExpr.equals(conditional.falseExpr);
    }
}
