package io.gitlab.jfronny.muscript.ast.dynamic;

import io.gitlab.jfronny.muscript.ast.DynamicExpr;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.DCallable;
import io.gitlab.jfronny.muscript.data.dynamic.additional.DFinal;
import io.gitlab.jfronny.muscript.data.dynamic.Dynamic;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Stream;

public class Bind extends DynamicExpr {
    public final DynamicExpr callable;
    public final DynamicExpr parameter;

    public Bind(CodeLocation location, DynamicExpr callable, DynamicExpr parameter) {
        super(Order.Call, location);
        this.callable = callable;
        this.parameter = parameter;
    }

    @Override
    public Dynamic get(Scope dataRoot) {
        Dynamic param = parameter.get(dataRoot);
        DCallable clb = callable.get(dataRoot).asCallable();
        return DFinal.of("<bind>", args -> {
            List<Dynamic> argsWithParameter = new LinkedList<>(args.getValue());
            argsWithParameter.add(0, param);
            return clb.call(DFinal.of(argsWithParameter));
        }, () -> this);
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        parenthesize(parameter, writer, false);
        writer.append("::");
        if (callable instanceof Variable) {
            callable.decompile(writer);
        } else {
            writer.append('(');
            callable.decompile(writer);
            writer.append(')');
        }
    }

    @Override
    public DynamicExpr optimize() {
        return new Bind(location, callable.optimize(), parameter.optimize());
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.concat(callable.extractSideEffects(), parameter.extractSideEffects());
    }
}
