package io.gitlab.jfronny.muscript.ast.dynamic;

import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.ast.DynamicExpr;
import io.gitlab.jfronny.muscript.annotations.CanThrow;
import io.gitlab.jfronny.muscript.annotations.UncheckedDynamic;
import io.gitlab.jfronny.muscript.error.LocationalException;

import java.io.IOException;
import java.util.stream.Stream;

@CanThrow
@UncheckedDynamic
public class Variable extends DynamicExpr {
    public final String name;

    public Variable(CodeLocation location, String name) {
        super(Order.Primary, location);
        this.name = name;
    }

    @Override
    public Dynamic get(Scope dataRoot) {
        if (name.equals("this")) return dataRoot;
        if (dataRoot.has(name)) return dataRoot.get(name);
        else throw new LocationalException(location, "This object doesn't contain '" + name + "'");
    }

    @Override
    public DynamicExpr optimize() {
        return this;
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.empty();
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        writer.appendLiteral(name);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Variable variable && name.equals(variable.name);
    }
}
