package io.gitlab.jfronny.muscript.ast.dynamic.assign;

import io.gitlab.jfronny.muscript.ast.*;
import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.DCallable;
import io.gitlab.jfronny.muscript.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.error.LocationalException;

import java.io.IOException;
import java.util.stream.Stream;

public class DynamicAssign extends DynamicExpr {
    private final String name;
    private final DynamicExpr value;

    public DynamicAssign(CodeLocation location, String name, DynamicExpr value) {
        super(Order.Primary, location);
        if (name.equals("this")) throw new LocationalException(location, "Cannot reassign 'this'");
        this.name = name;
        this.value = value;
    }

    @Override
    public Dynamic get(Scope dataRoot) {
        Dynamic data = value.get(dataRoot);
        dataRoot.set(name, data.isCallable() ? data.asCallable().named(name) : data);
        return data;
    }

    @Override
    public DynamicExpr optimize() {
        return new DynamicAssign(location, name, value.optimize());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        writer.appendLiteral(name).append(" = ");
        value.decompile(writer);
    }

    @Override
    public BoolExpr asBoolExpr() {
        return new BoolAssign(location, name, value.asBoolExpr());
    }

    @Override
    public NumberExpr asNumberExpr() {
        return new NumberAssign(location, name, value.asNumberExpr());
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.of(this);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof DynamicAssign assign
                && name.equals(assign.name)
                && value.equals(assign.value);
    }
}
