package io.gitlab.jfronny.muscript.ast.dynamic.assign;

import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.StringExpr;
import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.error.LocationalException;

import java.io.IOException;
import java.util.stream.Stream;

public class StringAssign extends StringExpr {
    private final String name;
    private final StringExpr value;

    protected StringAssign(CodeLocation location, String name, StringExpr value) {
        super(Order.Primary, location);
        if (name.equals("this")) throw new LocationalException(location, "Cannot reassign 'this'");
        this.name = name;
        this.value = value;
    }

    @Override
    public String get(Scope dataRoot) {
        String data = value.get(dataRoot);
        dataRoot.set(name, data);
        return data;
    }

    @Override
    public StringExpr optimize() {
        return new StringAssign(location, name, value.optimize());
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.of(this);
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        writer.appendLiteral(name).append(" = ");
        value.decompile(writer);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof StringAssign assign
                && name.equals(assign.name)
                && value.equals(assign.value);
    }
}
