package io.gitlab.jfronny.muscript.ast.dynamic.unpack;

import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.compiler.CodeLocation;
import io.gitlab.jfronny.muscript.compiler.ExprWriter;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.Dynamic;
import io.gitlab.jfronny.muscript.data.dynamic.DynamicTypeConversionException;
import io.gitlab.jfronny.muscript.ast.BoolExpr;
import io.gitlab.jfronny.muscript.ast.DynamicExpr;
import io.gitlab.jfronny.muscript.annotations.CanThrow;
import io.gitlab.jfronny.muscript.annotations.UncheckedDynamic;
import io.gitlab.jfronny.muscript.ast.dynamic.DynamicCoerce;
import io.gitlab.jfronny.muscript.error.LocationalException;

import java.io.IOException;
import java.util.stream.Stream;

@CanThrow
@UncheckedDynamic
public class BoolUnpack extends BoolExpr {
    public final DynamicExpr inner;

    public BoolUnpack(CodeLocation location, DynamicExpr inner) {
        super(inner.order, location);
        this.inner = inner;
    }

    @Override
    public Boolean get(Scope dataRoot) {
        try {
            Dynamic iv = inner.get(dataRoot);
            if (Dynamic.isNull(iv)) throw new LocationalException(location, "Cannot unpack null");
            return iv.asBool().getValue();
        } catch (DynamicTypeConversionException e) {
            throw e.locational(location);
        }
    }

    @Override
    public BoolExpr optimize() {
        DynamicExpr inner = this.inner.optimize();
        if (inner instanceof DynamicCoerce coerce) return coerce.inner.asBoolExpr();
        return new BoolUnpack(location, inner);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return inner.extractSideEffects();
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        inner.decompile(writer);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof BoolUnpack unpack && inner.equals(unpack.inner);
    }
}
