package io.gitlab.jfronny.muscript.ast.math;

import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.NumberExpr;
import io.gitlab.jfronny.muscript.ast.literal.NumberLiteral;

import java.io.IOException;
import java.util.stream.Stream;

public class Divide extends NumberExpr {
    public final NumberExpr dividend;
    public final NumberExpr divisor;

    public Divide(CodeLocation location, NumberExpr dividend, NumberExpr divisor) {
        super(Order.Factor, location);
        this.dividend = dividend;
        this.divisor = divisor;
    }

    @Override
    public Double get(Scope dataRoot) {
        return dividend.get(dataRoot) / divisor.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr dividend = this.dividend.optimize();
        NumberExpr divisor = this.divisor.optimize();
        if (dividend instanceof NumberLiteral litL && divisor instanceof NumberLiteral litR)
            return literal(location, litL.value / litR.value);
        if (dividend instanceof Divide divide && divide.dividend instanceof NumberLiteral literal)
            return new Divide(location, divide.dividend, new Multiply(divide.location, divisor, literal)).optimize();
        return new Divide(location, dividend, divisor);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.concat(dividend.extractSideEffects(), divisor.extractSideEffects());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        parenthesize(dividend, writer, false);
        writer.append(" / ");
        parenthesize(divisor, writer, true);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Divide divide && dividend.equals(divide.dividend) && divisor.equals(divide.divisor);
    }
}
