package io.gitlab.jfronny.muscript.ast.math;

import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.NumberExpr;
import io.gitlab.jfronny.muscript.ast.literal.NumberLiteral;

import java.io.IOException;
import java.util.stream.Stream;

public class Minus extends NumberExpr {
    public final NumberExpr minuend;
    public final NumberExpr subtrahend;

    public Minus(CodeLocation location, NumberExpr minuend, NumberExpr subtrahend) {
        super(Order.Term, location);
        this.minuend = minuend;
        this.subtrahend = subtrahend;
    }

    @Override
    public Double get(Scope dataRoot) {
        return minuend.get(dataRoot) - subtrahend.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr minuend = this.minuend.optimize();
        NumberExpr subtrahend = this.subtrahend.optimize();
        if (minuend instanceof NumberLiteral litM && subtrahend instanceof NumberLiteral litS)
            return literal(location, litM.value - litS.value);
        if (minuend instanceof Minus minus)
            return new Minus(location, minus.minuend, new Plus(minus.location, minus.subtrahend, subtrahend)).optimize();
        if (minuend instanceof Invert invM && subtrahend instanceof Invert invS)
            return new Minus(location, invS.inner, invM.inner);
        if (minuend instanceof Invert inv)
            return new Invert(location, new Plus(location, inv.inner, subtrahend)).optimize();
        if (subtrahend instanceof Invert inv) return new Plus(location, minuend, inv.inner).optimize();
        return new Minus(location, minuend, subtrahend);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.concat(minuend.extractSideEffects(), subtrahend.extractSideEffects());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        parenthesize(minuend, writer, false);
        writer.append(" - ");
        parenthesize(subtrahend, writer, true);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Minus minus && minuend.equals(minus.minuend) && subtrahend.equals(minus.subtrahend);
    }
}
