package io.gitlab.jfronny.muscript.ast.math;

import io.gitlab.jfronny.muscript.compiler.*;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.Expr;
import io.gitlab.jfronny.muscript.ast.NumberExpr;
import io.gitlab.jfronny.muscript.ast.literal.NumberLiteral;

import java.io.IOException;
import java.util.stream.Stream;

public class Multiply extends NumberExpr {
    public final NumberExpr multiplier;
    public final NumberExpr multiplicand;

    public Multiply(CodeLocation location, NumberExpr multiplier, NumberExpr multiplicand) {
        super(Order.Factor, location);
        this.multiplier = multiplier;
        this.multiplicand = multiplicand;
    }

    @Override
    public Double get(Scope dataRoot) {
        return multiplier.get(dataRoot) * multiplicand.get(dataRoot);
    }

    @Override
    public NumberExpr optimize() {
        NumberExpr multiplier = this.multiplier.optimize();
        NumberExpr multiplicand = this.multiplicand.optimize();
        if (multiplier instanceof NumberLiteral litEr && multiplicand instanceof NumberLiteral litAnd)
            return literal(location, litEr.value * litAnd.value);
        return new Multiply(location, multiplier, multiplicand);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return Stream.concat(multiplier.extractSideEffects(), multiplicand.extractSideEffects());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        parenthesize(multiplier, writer, false);
        writer.append(" * ");
        parenthesize(multiplicand, writer, true);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof Multiply multiply && multiplier.equals(multiply.multiplier) && multiplicand.equals(multiply.multiplicand);
    }
}
