package io.gitlab.jfronny.muscript.ast.string;

import io.gitlab.jfronny.commons.StringFormatter;
import io.gitlab.jfronny.muscript.compiler.CodeLocation;
import io.gitlab.jfronny.muscript.compiler.ExprWriter;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.ast.*;
import io.gitlab.jfronny.muscript.ast.literal.*;

import java.io.IOException;
import java.util.stream.Stream;

public class StringCoerce extends StringExpr {
    private final Expr<?> inner;

    public StringCoerce(CodeLocation location, Expr<?> inner) {
        super(inner.order, location);
        this.inner = inner;
    }

    @Override
    public String get(Scope dataRoot) {
        return StringFormatter.toString(inner.get(dataRoot));
    }

    @Override
    public StringExpr optimize() {
        Expr<?> inner = this.inner.optimize();
        if (inner instanceof NullLiteral) return literal(location, "null");
        if (inner instanceof BoolLiteral expr) return literal(location, StringFormatter.toString(expr.value));
        if (inner instanceof DynamicLiteral<?> expr) return literal(location, StringFormatter.toString(expr.value));
        if (inner instanceof NumberLiteral expr) return literal(location, StringFormatter.toStringPrecise(expr.value));
        if (inner instanceof StringExpr expr) return expr;
        return new StringCoerce(location, inner);
    }

    @Override
    public Stream<Expr<?>> extractSideEffects() {
        return inner.extractSideEffects();
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        inner.decompile(writer);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof StringCoerce coerce && inner.equals(coerce.inner);
    }
}
